/*
 *  
 *  Copyright (c) 2004 Steve Slaven, All Rights Reserved.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA 02111-1307 USA
 *  
*/
// Just decodes hashes
package net.hoopajoo.cheeselab3;
import java.util.*;
import java.io.*;
import java.sql.*;

class CheeseDecoder {
	public static HashMap decode_hash( char[] r ) {
		String s = new String( r );

		BufferedReader in = new BufferedReader( new StringReader( s ) );
		HashMap m = new HashMap();

		String line;
		try{
			while( (line = in.readLine()) != null ) {
				int colon = line.indexOf( ":" );
				String key = line.substring( 0, colon );
				String value = line.substring( colon + 1 );
				//CheeseMap.dmesg( "Setting '" + key + "' to " + value );
				m.put( key, value );
			}
		}catch( Exception e ){
		}

		return( m );
	}
	
	public static Object decode_object( byte[] input ) {
		CheeseLab.dmesg( 2, "Decoding object " );
		Object ret = null;
		try {
			ObjectInputStream s = new ObjectInputStream(
				new ByteArrayInputStream( input ) );
			ret = s.readObject();
			s.close();
			CheeseLab.dmesg( 2, "Got class: " + ret.getClass().getName() );
		}catch( Exception e ) {
			e.printStackTrace();
		}
		
		
		return( ret );
	}
	
	public static byte[] encode_object( Object input ) {
		byte[] ret = null;
		try {
			ByteArrayOutputStream bytes = new ByteArrayOutputStream();
			ObjectOutputStream s = new ObjectOutputStream( bytes );
			s.writeObject( input );
			s.close();
			ret = bytes.toByteArray();
		}catch( Exception e ) {
			e.printStackTrace();
		}
		return( ret );
	}
	
	public static void room_to_sql( CheeseRoomData r, Connection c, String userid ) {
		try {
			PreparedStatement s = c.prepareStatement(
				"update room_t " +
				"set notes=?, overlays=?," +
				"	name=?, edesc=?, terrain=?," +
				"	x=?, y=?, z=?," +
				"	exit_n=?, exit_s=?, exit_e=?, exit_w=?, exit_u=?, exit_d=?," +
				"	door_n=?, door_s=?, door_e=?, door_w=?, door_u=?, door_d=?," +
				"	userid=?, mtime=?" +
				"where roomid=?"
			);
			int col = 1;
			String overlays = "";
			for( int i = 0; i < r.overlays.length; i++ ) {
				if( i != 0 ) 
					overlays += ",";
				overlays += r.overlays[ i ];
			}
			s.setString( col++, r.notes ); s.setString( col++, overlays );
			
			s.setString( col++, r.name ); s.setString( col++, r.edesc ); s.setString( col++, r.terrain );

			s.setInt( col++, r.x );	s.setInt( col++, r.y );	s.setInt( col++, r.z );
			
			s.setInt( col++, r.exit_n );
			s.setInt( col++, r.exit_s );
			s.setInt( col++, r.exit_e );
			s.setInt( col++, r.exit_w );
			s.setInt( col++, r.exit_u );
			s.setInt( col++, r.exit_d );
			
			s.setString( col++, r.door_n );
			s.setString( col++, r.door_s );
			s.setString( col++, r.door_e );
			s.setString( col++, r.door_w );
			s.setString( col++, r.door_u );
			s.setString( col++, r.door_d );
			
			s.setString( col++, userid );
			s.setLong( col++, System.currentTimeMillis() );

			s.setInt( col++, r.roomid );
			
			s.executeUpdate();
			s.close();
		}catch( Exception e ) {
			e.printStackTrace();
		}
	}
	
	public static CheeseRoomData get_room( int roomid, Connection c ) {
		CheeseRoomData r = null;
		try{
			PreparedStatement p = c.prepareStatement( "select * from room_t where roomid=?" );
			p.setInt( 1, roomid );
			ResultSet rs = p.executeQuery();
			if( rs.next() ) {
				r = sql_to_room( rs );
			}
			rs.close();
			p.close();
		}catch( Exception e )  {
			e.printStackTrace();
		}
		return( r );
	}
	
	public static CheeseRoomData sql_to_room( ResultSet rs ) {
		CheeseRoomData n = new CheeseRoomData();
		
		try {
			n.roomid = rs.getInt( "roomid" );
			n.name = rs.getString( "name" );
			n.edesc = rs.getString( "edesc" );
			n.terrain = rs.getString( "terrain" );
			n.notes = rs.getString( "notes" );
	
			n.userid = rs.getString( "userid" );
			n.mtime = rs.getLong( "mtime" );

			n.x = rs.getInt( "x" );
			n.y = rs.getInt( "y" );
			n.z = rs.getInt( "z" );
			
			n.exit_n = rs.getInt( "exit_n" );
			n.exit_s = rs.getInt( "exit_s" );
			n.exit_e = rs.getInt( "exit_e" );
			n.exit_w = rs.getInt( "exit_w" );
			n.exit_u = rs.getInt( "exit_u" );
			n.exit_d = rs.getInt( "exit_d" );
			
			n.door_n = rs.getString( "door_n" );
			n.door_s = rs.getString( "door_s" );
			n.door_e = rs.getString( "door_e" );
			n.door_w = rs.getString( "door_w" );
			n.door_u = rs.getString( "door_u" );
			n.door_d = rs.getString( "door_d" );
			
			String overlays = rs.getString( "overlays" );
			if( overlays == null )
				overlays = new String();
			
			n.overlays = overlays.split( "," );
		}catch( Exception e ) {
			e.printStackTrace();
		}
		
		return( n );
	}
	
	public static CheesePlayerData get_player( String userid, Connection db ) {
		CheesePlayerData p = new CheesePlayerData();
		
		try {
			PreparedStatement s = db.prepareCall( "select * from user_t where userid=?" );
			s.setString( 1, userid );
			ResultSet rs = s.executeQuery();
			
			if( rs.next() ) {
				p.name = rs.getString( "userid" );
				
				p.x = rs.getInt( "x" );
				p.y = rs.getInt( "y" );
				p.z = rs.getInt( "z" );
				
				p.hit_curr = rs.getInt( "hit_curr" );
				p.hit_max = rs.getInt( "hit_max" );
				p.mana_curr = rs.getInt( "mana_curr" );
				p.mana_max = rs.getInt( "mana_max" );
				p.move_curr = rs.getInt( "move_curr" );
				p.move_max = rs.getInt( "move_max" );
				
				p.preferences = rs.getString( "preferences" );
				p.permissions = rs.getString( "permissions" );
				
				p.roomid = rs.getInt( "roomid" );
				p.online = rs.getInt( "online" ) == 1;
				
				p.future = rs.getString( "future" ).split( "-" );
				
				rs.close();
				s.close();
			}else{
				rs.close();
				s.close();
				return( null );
			}
		}catch( Exception e ) {
			e.printStackTrace();
		}
		
		return( p );
	}
}
